package container;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 协议采集到数据的容器
 *
 * @author 修唯xiuwei
 * @version 3.0
 */

@Slf4j
public class ProtocolDataContainer_Local extends ProtocolDataContainer {


    /**
     * 遥测数据
     */
    private Map<String, BigDecimal> yc = new ConcurrentHashMap<>();
    /**
     * 遥信数据
     */
    private Map<String, Boolean> yx = new ConcurrentHashMap<>();


    private ProtocolDataContainer_Local() {
    }

    /**
     * 获取单例实例
     *
     * @return the instance
     */
    public static final ProtocolDataContainer_Local getInstance() {
        return LazyHolder.INSTANCE;
    }

    /**
     * 向池子内置数据
     *
     * @param point point
     * @param value value
     */
    public void putNumber(String point, BigDecimal value) {
        if (point == null || point == "") {
            throw new IllegalArgumentException("存入数据池数据，入参点位为空");
        }
        if (value == null) {
            throw new IllegalArgumentException("存入数据池数据，入值点位为空");
        }
        this.yc.put(point, value.setScale(2, RoundingMode.HALF_UP));
    }



    /**
     * 向池子内置数据
     *
     * @param point point
     * @param value value
     */
    public void putBoolean(String point, Boolean value) {
        this.yx.put(point, value);
    }




    /**
     * 获取到缓存池内的数值 如果遥测池内没有但是存在于遥信池 将返回 0or1
     *
     * @param point 点位
     * @return Number number
     */
    public BigDecimal getNumber(String point) {
        if (isYcContains(point)) {
            return this.yc.get(point);
        } else if (isYxContains(point)) {
            return this.yx.get(point) ? BigDecimal.valueOf(1D) : BigDecimal.valueOf(0D);
        } else {
            return BigDecimal.valueOf(-99D);
        }
    }

    /**
     * 获取到返回池内的遥信数据  如果遥信池内没有 遥测池内有 那么大于0的数值 将会返回ture 否则为false
     *
     * @param point 点位
     * @return Boolean boolean
     */
    public Boolean getBoolean(String point) {
        if (isYxContains(point)) {
            return this.yx.get(point);
        } else if (isYcContains(point)) {
            return this.yc.get(point).doubleValue() > 0;
        } else {
            return false;
        }
    }

    /**
     * 遥信数据池是否含有该点位
     *
     * @param point point
     * @return the boolean
     */
    public Boolean isYxContains(String point) {
        return this.yx.containsKey(point);
    }

    /**
     * 遥测数据池是否含有该点位值
     *
     * @param point point
     * @return the boolean
     */
    public Boolean isYcContains(String point) {
        return this.yc.containsKey(point);
    }


    @Override
    public String toString() {
        JSONObject object = new JSONObject();
        object.put("yx", this.yx);
        object.put("yc", this.yc);
        return object.toJSONString();
    }

    public Map<String, BigDecimal> getYcMap() {
        return this.yc;
    }

    public Map<String, Boolean> getYxMap() {
        return this.yx;
    }


    private static class LazyHolder {
        private static final ProtocolDataContainer_Local INSTANCE = new ProtocolDataContainer_Local();
    }

}
