package datapacker.iml;


import container.ProtocolDataContainer;
import datapacker.AbstractDataPacker;
import entity.BaseRealtimeData;
import entity.datapoint.BaseDataPoint;
import exception.DataExchangeException;
import lombok.Getter;
import lombok.Setter;
import org.joda.time.DateTime;

import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.util.*;

/**
 * 装备数据的包装类 点表转类属性
 *
 * @author 修唯xiuwei
 * @version 3.0
 */
@Getter
public class EquipmentDataPacker extends AbstractDataPacker {

    /**
     * 设备号
     */
    @Setter
    Integer equipmentNo;

    /**
     * 实时采集数据池
     */
    ProtocolDataContainer protocolDataContainerRedis = ProtocolDataContainer.getInstance();

    /**
     * 所有需要采集的数据点
     */
    private List<BaseDataPoint> pointList;

    /**
     * 将点位与类的属性对应上的map
     */
    private Map<BaseDataPoint, Field> point2FiledMap = new HashMap<>();

    /**
     * 构造方法，在方法中获取传入类型的属性，存入map中，以便后期使用
     *
     * @param newInstance T的实例 直接new 对象（）；
     * @param pointList   点表集合
     */
    public EquipmentDataPacker(BaseRealtimeData newInstance, List<BaseDataPoint> pointList) {
        setRealtimeData(newInstance);
        if (pointList == null || pointList.size() == 0) {
            return;
        }
        parseBaseDataPoint(pointList);
    }

    /**
     * 解析采集数据点
     *
     * @param pointList 点列表
     */
    public void parseBaseDataPoint(List<BaseDataPoint> pointList) {
        this.pointList = pointList;
        this.equipmentNo = pointList.get(0).getEquipmentNo();
        point2FiledMap = new HashMap<>();
        List<Field> fields = new ArrayList<>();
        fields.addAll(Arrays.asList(getRealtimeData().getClass().getDeclaredFields()));
        fields.addAll(Arrays.asList(BaseRealtimeData.class.getDeclaredFields()));
        for (Field f : fields) {
            for (BaseDataPoint p : pointList) {
                if (f.getName().equals(p.getEquipmentAttribute().getFieldName())) {
                    f.setAccessible(true);
                    point2FiledMap.put(p, f);
                }
            }
        }
    }

    /**
     * 通过反射机制将容器中的数据封装入设备描述的实体类 同时对数据合理性进行校验 仅对数据类型为BigDecimal的数据进行合理性校验
     *
     * @return AbstractEquipmentData 设备状态数据
     * @throws DataExchangeException 异常
     */
    @Override
    public synchronized BaseRealtimeData packageData() throws DataExchangeException {
        BaseRealtimeData data;
        try {
            data = realtimeData.getClass().newInstance();
        } catch (InstantiationException e) {
            throw new DataExchangeException(40101, "传入实体类实例化失败");
        } catch (IllegalAccessException e) {
            throw new DataExchangeException(40102, "传入实体类构建实例时权限异常");
        }
        Type type;
        if (this.point2FiledMap == null) {
            return data;
        }
        for (Map.Entry<BaseDataPoint, Field> entry : point2FiledMap.entrySet()) {
            String dpId = "";
            try {
                dpId = entry.getKey().getSId();
                if (protocolDataContainerRedis.isYxContains(dpId)
                    || protocolDataContainerRedis.isYcContains(dpId)) {
                    type = entry.getValue().getGenericType();
                    if (Boolean.class.equals(type) || boolean.class.equals(type)) {
                        entry.getValue().set(data, protocolDataContainerRedis.getBoolean(dpId));
                    } else if (BigDecimal.class.equals(type)) {
                        entry.getValue().set(data,
                            protocolDataContainerRedis.getNumber(dpId).setScale(2, BigDecimal.ROUND_HALF_UP));
                    } else if (Integer.class.equals(type) || int.class.equals(type)) {
                        entry.getValue().set(data, protocolDataContainerRedis.getNumber(dpId).intValue());
                    }
                }
            } catch (IllegalAccessException e) {
                e.printStackTrace();
                throw new DataExchangeException(40103, "传入实体域注入值时权限异常");
            } catch (IllegalArgumentException e) {
                e.printStackTrace();
                throw new DataExchangeException(40104, "传入实体域注入值时类型异常");
            }
        }
        data.setEquipmentNo(equipmentNo);
        data.setTime(DateTime.now().withMillisOfSecond(0).withSecondOfMinute(0).toDate());
        return data;
    }

}
