package entity.datapoint;

import com.alibaba.fastjson.JSON;
import com.googlecode.aviator.AviatorEvaluator;
import com.googlecode.aviator.Expression;
import com.googlecode.aviator.Options;
import com.googlecode.aviator.exception.ExpressionSyntaxErrorException;

import container.ProtocolDataContainer;
import enums.DataType;
import exception.DataExchangeException;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 计算点位的计算器
 *
 * @author: xiuwei
 * @version:
 */
@Slf4j
public class CalculatorOfDataPoint {


	private static final Pattern expr = Pattern.compile("(?<=P_).*?(?=_P)");

	static {
		AviatorEvaluator.setOption(Options.ALWAYS_PARSE_FLOATING_POINT_NUMBER_INTO_DECIMAL, true);
	}

	/**
	 * 计算表达式
	 */
	Expression compiledExp;
	/**
	 * 数据类型
	 */
	DataType dataType;
	Integer id;
	/**
	 * 计算所需参数
	 */
	private Map<String, String> calculatingParameters;
	/**
	 * 计算所需公式
	 */
	@Getter
	private String formula;
	private ProtocolDataContainer protocolDataContainerRedis = ProtocolDataContainer.getInstance();

	public CalculatorOfDataPoint(CalculateDataPoint calculateDataPoint) {
		this.id = calculateDataPoint.getId();
		this.dataType = calculateDataPoint.getDataType();
		this.formula = calculateDataPoint.getFormula();
	}

	/**
	 * 计算
	 * 对公式进行计算
	 *
	 * @return V 计算完成的数据
	 * @throws DataExchangeException 数据交换异常
	 */
	public void calculate() throws DataExchangeException {
		Map<String, Object> values = new HashMap<>();
		for (Map.Entry<String, String> e : getCalculatingParameters().entrySet()) {
			if (protocolDataContainerRedis.isYcContains(e.getValue())) {
				values.put(e.getKey(), protocolDataContainerRedis.getNumber(e.getValue()));
			} else if (protocolDataContainerRedis.isYxContains(e.getValue())) {
				values.put(e.getKey(), protocolDataContainerRedis.getNumber(e.getValue()));
			} else {
				log.error("数据池内无匹配点位：" + e.getValue());
				return;
			}
		}
		try {
			if (this.dataType == DataType.YX) {
				protocolDataContainerRedis.putBoolean("J" + this.id, (Boolean) (getOrInitCompiledExp().execute(values)));
			} else {
				protocolDataContainerRedis.putNumber("J" + this.id, (BigDecimal) (getOrInitCompiledExp().execute(values)));
			}
		} catch (ArithmeticException e) {
			log.error("除数为0,计算公式为：{},请检查数据池内是否有该点位是否有数据", this.formula);
		} catch (Exception e) {
			log.error("数据计算{}时发生异常", this.formula, e);
			log.info(JSON.toJSONString(values));
			throw new DataExchangeException(10021, "计算器计算数据时发生异常,公式为：" + this.formula);
		}
	}

	/**
	 * 对公式进行初始化  如果公式解析失败将会抛出异常
	 *
	 * @return Expression  解析完成的表达式
	 * @throws DataExchangeException 表达式解析失败时的异常
	 */
	private Expression getOrInitCompiledExp() throws DataExchangeException {
		if (this.compiledExp == null) {
			try {
				this.compiledExp = AviatorEvaluator.compile(formula);
			} catch (ExpressionSyntaxErrorException e) {
				log.error("公式解析失败，公式不可用", e);
				throw new DataExchangeException(50001, "公式解析失败，公式不可用");
			}
		}
		return this.compiledExp;
	}


	/**
	 * 对计算所需的参数进行整理
	 *
	 * @return P_（点位）_P  替换成真实的数据   字段--数值
	 */
	public Map<String, String> getCalculatingParameters() {
		if (this.calculatingParameters == null) {
			this.calculatingParameters = new HashMap<>();
			Matcher matcher = expr.matcher(this.formula);
			while (matcher.find()) {
				String s = matcher.group();
				this.calculatingParameters.put("P_" + s + "_P", s);
			}
		}
		return this.calculatingParameters;
	}

}
