package tunnelworker.masters.iml;


import ch.qos.logback.classic.Logger;

import entity.datapoint.CommandDataPoint;
import entity.datapoint.GatherDataPoint;
import enums.CommandProtocolType;
import enums.TunnelStatus;
import exception.DataExchangeException;
import mastertunnelinfo.Master104TcpTunnelInfo;
import org.slf4j.LoggerFactory;
import tunnelworker.BaseProtocolTunnel;
import tunnelworker.masters.MasterInterface;
import tunnelworker.workassist.SingleThreadPoolExecutorUtil;
import wei.yigulu.iec104.nettyconfig.Iec104HSMasterBuilder;
import wei.yigulu.iec104.util.SendCommandHelper;
import wei.yigulu.iec104.util.SendDataFrameHelper;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 * 104协议数据获取
 *
 * @author 修唯xiuwei
 * @version 3.0
 */

public class Iec104TcpMaster extends BaseProtocolTunnel<Master104TcpTunnelInfo, Iec104HSMasterBuilder> implements MasterInterface {

    private static final Logger logger = (Logger) LoggerFactory.getLogger(Iec104TcpMaster.class);

    /**
     * 点表
     */
    private Map<Integer, GatherDataPoint> iec104DataPoint = new HashMap<>();


    /**
     * iec104tcp数据采集
     * 构造方法
     *
     * @param gather104TcpTunnelInfo 104采集通道信息
     * @throws DataExchangeException 数据交换异常
     */
    public Iec104TcpMaster(Master104TcpTunnelInfo gather104TcpTunnelInfo) throws DataExchangeException {
        super(gather104TcpTunnelInfo);
        parseDataPoint();
    }

    @Override
    protected void parseDataPoint() throws DataExchangeException {
        List<GatherDataPoint> dataPoints = this.getTunnelInfo().getDataPoints();
        iec104DataPoint = new HashMap<>();
        if (dataPoints == null || dataPoints.size() == 0) {
            return;
        }
        for (GatherDataPoint d : dataPoints) {
            this.iec104DataPoint.put(d.getProtocolPoint(), d);
        }
        if (protocolBuilder != null) {
            protocolBuilder.getConfigInfoMap().put("104DataPoint", this.iec104DataPoint);
        }
    }


    @Override
    protected Iec104TcpMaster buildTunnel() throws DataExchangeException {
        protocolBuilder = new MyMasterBuilder(tunnelInfo.getRemoteIp(), tunnelInfo.getRemotePort());
        protocolBuilder.getConfigInfoMap().put("tunnelId", this.tunnelInfo.getId());
        this.protocolBuilder.getConfigInfoMap().put("tunnelName", this.tunnelInfo.getTunnelName());
        protocolBuilder.getConfigInfoMap().put("104DataPoint", this.iec104DataPoint);
        protocolBuilder.setLog(log);
        if (getTunnelInfo().getRemoteSpareIp() != null) {
            protocolBuilder.setSpareIp(getTunnelInfo().getRemoteSpareIp());
        }
        if (getTunnelInfo().getRemoteSparePort() != null) {
            protocolBuilder.setSparePort(getTunnelInfo().getRemoteSparePort());
        }
        if (getTunnelInfo().getSelfIp() != null) {
            protocolBuilder.setSelfIp(getTunnelInfo().getSelfIp());
        }
        if (getTunnelInfo().getSelfPort() != null) {
            protocolBuilder.setSelfPort(getTunnelInfo().getSelfPort());
        }
        getProtocolTunnelContainer().addTunnel(this);
        this.setTunnelStatus(TunnelStatus.BUILT);
        log.info("成功创建Iec 104Tcp master通道对象：{}", this.tunnelInfo.getTunnelName());
        return this;
    }

    @Override
    public Iec104TcpMaster startTunnel() throws DataExchangeException {
        protocolTunnelContainer.addUpdateDateTask(this);
        try {
            SingleThreadPoolExecutorUtil.executeBySingleThreadExecutor(() -> this.protocolBuilder.create());
        } catch (Exception e) {
            throw new DataExchangeException(10009, "104master通道创建时发生异常");
        }
        log.info("IEC104 TCP master {} 通道开始连接", getTunnelInfo().getTunnelName());
        return this;
    }


    @Override
    public BaseProtocolTunnel tunnelStop() throws DataExchangeException {
        if (this.protocolBuilder != null) {
            this.protocolBuilder.stop();
        }
        log.info("104 master {} 关闭", tunnelInfo.getTunnelName());
        return super.tunnelStop();
    }

    @Override
    public void getData() throws DataExchangeException {
        log.info("向104 通道:{} 中发送总召唤", getTunnelInfo().getTunnelName());
        try {
            if (this.protocolBuilder.getFuture().channel() != null && this.protocolBuilder.getFuture().channel().isActive()) {
                SendDataFrameHelper.sendTotalSummonFrame(this.protocolBuilder.getFuture().channel(), getTunnelInfo().getPublicAddress(), 6, this.log);
            }
        } catch (Exception e) {
            log.error("104通道发送数据时发生异常", e);
            throw new DataExchangeException(10008, "104通道发送数据时发生异常");
        }
    }

    @Override
    public boolean sendCommand(CommandDataPoint dataPoint, Object val) throws Exception {
        logger.debug("通道{}发送控制命令，命令类型{},命令地址{},控制值{}", tunnelInfo.getTunnelName(), dataPoint.getCommandProtocolType().getExplain(), dataPoint.getProtocolPoint(), val);
        logger.debug("当下通道状态：{}", getTunnelStatus());
        if (dataPoint.getCommandProtocolType() == CommandProtocolType.SHORT_FLOAT) {
            return SendCommandHelper.sendShortCommand(protocolBuilder, 0, tunnelInfo.getPublicAddress(), dataPoint.getProtocolPoint(), ((BigDecimal) val).multiply(dataPoint.getRatio()).floatValue());
        } else {
            return false;
        }
    }


    /**
     * 内部类 用于设置连接 和断连 的后续动作
     */
    class MyMasterBuilder extends Iec104HSMasterBuilder {
        public MyMasterBuilder(String ip, Integer port) {
            super(ip, port);
        }


        @Override
        public void connected() {
            log.info("IEC104 Master {}连接成功", getTunnelInfo().getTunnelName());
            setTunnelStatus(TunnelStatus.CONNECTED);
            try {
                Executors.newSingleThreadScheduledExecutor().schedule(() -> {
                    try {
                        getData();
                    } catch (DataExchangeException e) {
                        log.error("通道连接时发送总召唤异常");
                    }
                }, 3L, TimeUnit.SECONDS);
            } catch (Exception e) {
                log.error("通道连接时发送总召唤异常");
            }
        }

        @Override
        public void disconnected() {
            log.error("IEC104 Master {}断开连接", getTunnelInfo().getTunnelName());
            setTunnelStatus(TunnelStatus.LOSECONN);
        }


    }

}
